"""Tests the K2findCampaigns module."""
import tempfile

from .. import K2findCampaigns


def test_find_campaigns_basics():
    campaigns = K2findCampaigns.findCampaigns(269.5, -28.5)
    assert(9 in campaigns)
    campaigns = K2findCampaigns.findCampaigns(0, 0)
    assert(campaigns == [])


def test_K2findCampaigns_csv():
    """Test the csv version."""
    csv = '269.5, -28.5, 0\n0, 0, 0\n'
    with tempfile.NamedTemporaryFile() as temp:
        try:
            # Python 3
            temp.write(bytes(csv, 'utf-8'))
        except TypeError:
            # Legacy Python
            temp.write(csv)
        temp.flush()
        K2findCampaigns.K2findCampaigns_csv_main(args=[temp.name])



"""Test whether the `K2onSilicon` command-line tool works."""
import tempfile
import numpy as np

from ..K2onSilicon import K2onSilicon_main


def test_K2onSilicon():
    """Test the basics: does K2onSilicon run without error on a dummy file?"""
    csv = '269.5, -28.5, 12\n0, 0, 20\n'
    with tempfile.NamedTemporaryFile() as temp:
        try:
            # Python 3
            temp.write(bytes(csv, 'utf-8'))
        except TypeError:
            # Legacy Python
            temp.write(csv)
        temp.flush()
        K2onSilicon_main(args=[temp.name, "9"])

    # Verify the output
    output_fn = "targets_siliconFlag.csv"
    ra, dec, mag, status = np.atleast_2d(
                    np.genfromtxt(
                                output_fn,
                                usecols=[0, 1, 2, 3],
                                delimiter=','
                                )
            ).T
    # The first target is one silicon in C9, the second is not
    assert(int(status[0]) == 2)
    assert(int(status[1]) == 0)
    # Sanity check of the other columns
    assert(ra[0] == 269.5)
    assert(dec[0] == -28.5)
    assert(mag[0] == 12)
    assert(ra[1] == 0)
    assert(dec[1] == 0)
    assert(mag[1] == 20)



"""Sanity check: are all entries of an observed target list on silicon?"""
import os
import numpy as np

from .. import getKeplerFov
from ..K2onSilicon import onSiliconCheck, onSiliconCheckList

# Where is this test script located?
TESTDIR = os.path.dirname(os.path.abspath(__file__))


def test_targetlists():

    def run_test(campaign):
        """Are entries in the target list of a given campaign on silicon?"""
        fov = getKeplerFov(campaign)
        if campaign < 11:
            # Module 4 didn't break until Campaign 10,
            # whereas Modules 3 and 7 failed before Campaign 0.
            fov.brokenChannels = [5, 6, 7, 8, 17, 18, 19, 20]
        targetlist_fn = os.path.join(TESTDIR,
                                     "data",
                                     "K2Campaign{0}targets.csv".format(campaign))
        if campaign == 9:
            targetlist_fn = targetlist_fn.replace('9', '9a')
        targetlist = np.genfromtxt(targetlist_fn,
                                   delimiter=",", dtype=None, names=True)
        ra, dec = targetlist["RA_J2000_deg"], targetlist["Dec_J2000_deg"]
        mask = ~np.isnan(ra) & ~np.isnan(dec)

        for idx in np.where(mask)[0][::500]:  # Speed-up: check every 500th
            # All the sources in the target list should be on silicon
            assert(onSiliconCheck(ra[idx], dec[idx], fov))
            assert(not onSiliconCheck(ra[idx] + 20, dec[idx], fov))

        # Also check whether the list-checking function works correctly
        try:
            idx = np.where(mask)[0][::500]
            assert(np.all(onSiliconCheckList(ra[idx], dec[idx], fov)))
            assert(np.all(~onSiliconCheckList(ra[idx] + 20, dec[idx], fov)))
        except ImportError:
            pass  # This feature requires AstroPy, which is an opional requirement

    # We test all the target lists available at the time of writing this test
    for campaign in range(15):
        run_test(campaign)


if __name__ == "__main__":
    test_targetlists()



"""Tests the functionality specific to the K2C9 microlensing campaign."""
from .. import c9


def test_point_inside_polygon():
    """Basic test of c9.isPointInsidePolygon()"""
    vertices_x = [+1, +1, -1, -1]
    vertices_y = [+1, -1, -1, +1]
    # Simple cases
    for x, y in [(0, 0), (0.5, -0.5), (-0.5, 0.5)]:
        assert(c9.isPointInsidePolygon(x, y, vertices_x, vertices_y))
    for x, y in [(2, 2), (1.5, -0.5), (0.5, 1.5)]:
        assert(not c9.isPointInsidePolygon(x, y, vertices_x, vertices_y))
    # Edge cases
    for x, y in [(0.999, 0.999), (0.999, -0.999), (-0.999, 0.999)]:
        assert(c9.isPointInsidePolygon(x, y, vertices_x, vertices_y))
    for x, y in [(1.001, 0), (0, 1.001), (-1.001, 0.999)]:
        assert(not c9.isPointInsidePolygon(x, y, vertices_x, vertices_y))


def test_in_microlens_region():
    """Basic test of c9.inMicrolensRegion()"""
    # The following coordinates are well within the superstamp
    # according to plots circulated by Radek on the K2C9 mailing list
    for ra, dec in [(269.5, -28.5), (271, -28.2), (268.5, -28.5),
                    (269.9, -27.5), (271.2, -27.2), (269.2, -28.8)]:
        assert(c9.inMicrolensRegion(ra, dec))
        # All coordinates 5 degrees away are not in the superstamp
        assert(not c9.inMicrolensRegion(ra - 5, dec))
        assert(not c9.inMicrolensRegion(ra + 5, dec))
        assert(not c9.inMicrolensRegion(ra, dec - 5))
        assert(not c9.inMicrolensRegion(ra, dec + 5))

    # The coordinates below are also definitely not inside the region
    for ra, dec in [(0, 0), (0, +90), (90, -45), (270, +45)]:
        assert(not c9.inMicrolensRegion(ra, dec))




import numpy as np
import unittest
from .. import rotate2 as r

#$Id: test_rotate2.py 28 2013-12-02 04:34:57Z fergalm $
#$URL: svn+ssh://fergalm@svn.code.sf.net/p/keplertwowheel/code/py/test/test_rotate2.py $

class TestSingleRotation(unittest.TestCase):

    def testXAroundZ(self):
        for deg in np.arange(0, 360):
            ct = np.cos( np.radians(deg))
            st = np.sin( np.radians(deg))

            v = np.array([1,0,0])
            exp = np.array([ ct, st, 0])

            Rmat = r.rotateInZMat(deg)
            calc = np.dot(Rmat, v)

            msg = "Angle %.0f: Expect %s, Calc %s" %(deg, exp, calc)
            for i in range(3):
                self.assertAlmostEqual(exp[i], calc[i], 6, msg)


    def testYAroundZ(self):
        for deg in np.arange(0, 360):
            ct = np.cos( np.radians(deg))
            st = np.sin( np.radians(deg))

            v = np.array([0,1,0])
            exp = np.array([ -st, ct, 0])

            Rmat = r.rotateInZMat(deg)
            calc = np.dot(Rmat, v)

            msg = "Angle %.0f: Expect %s, Calc %s" %(deg, exp, calc)
            for i in range(3):
                self.assertAlmostEqual(exp[i], calc[i], 6, msg)


    def testXAroundY(self):
        for deg in np.arange(0, 360):
            ct = np.cos( np.radians(deg))
            st = np.sin( np.radians(deg))

            v = np.array([1,0,0])
            exp = np.array([ ct, 0, -st])

            Rmat = r.rotateInYMat(deg)
            calc = np.dot(Rmat, v)

            msg = "Angle %.0f deg:\n Expect %s, Calc %s" \
                %(deg, exp, calc)
            for i in range(3):
                self.assertAlmostEqual(exp[i], calc[i], 6, msg)


    def testZAroundY(self):
        for deg in np.arange(0, 360):
            ct = np.cos( np.radians(deg))
            st = np.sin( np.radians(deg))

            v = np.array([0,0,1])
            exp = np.array([ st, 0, ct])

            Rmat = r.rotateInYMat(deg)
            calc = np.dot(Rmat, v)

            msg = "Angle %.0f deg:\n Expect %s, Calc %s" \
                %(deg, exp, calc)
            for i in range(3):
                self.assertAlmostEqual(exp[i], calc[i], 6, msg)


class TestRotationOrder(unittest.TestCase):


    def testSmoke(self):
        self.anglePairTestX(30,60)
        self.anglePairTestY(30,60)

    def XtestAll(self):
        for ang1 in range(0, 361, 10):
            for ang2 in range(-90, 91, 10):
                self.anglePairTest(ang1, ang2)


    def anglePairTestX(self, theta_deg, phi_deg):
        ct = np.cos( np.radians(theta_deg))
        st = np.sin( np.radians(theta_deg))

        cp = np.cos( np.radians(phi_deg))
        sp = np.sin( np.radians(phi_deg))


        e1 = ct*cp
        e2 = st
        e3 = -ct*sp

        x = np.array( [1,0,0])
        exp = np.array( [e1, e2, e3] )

        Rz = r.rotateInZMat(theta_deg)
        Ry = r.rotateInYMat(phi_deg)

        rotMat = np.dot(Ry, Rz)
        calc = np.dot(rotMat, x)

        msg = "Angles %.0f %.0f: Expect %s, Calc %s" \
            %(theta_deg, phi_deg, exp, calc)
        for i in range(3):
            self.assertAlmostEqual(exp[i], calc[i], 6, msg)



    def anglePairTestY(self, theta_deg, phi_deg):
        ct = np.cos( np.radians(theta_deg))
        st = np.sin( np.radians(theta_deg))

        cp = np.cos( np.radians(phi_deg))
        sp = np.sin( np.radians(phi_deg))


        e1 = -st*cp
        e2 = ct
        e3 = st*sp

        y = np.array( [0,1,0])
        exp = np.array( [e1, e2, e3] )

        Rz = r.rotateInZMat(theta_deg)
        Ry = r.rotateInYMat(phi_deg)

        rotMat = np.dot(Ry, Rz)
        calc = np.dot(rotMat, y)

        msg = "Angles %.0f %.0f: Expect %s, Calc %s" \
            %(theta_deg, phi_deg, exp, calc)
        for i in range(3):
            self.assertAlmostEqual(exp[i], calc[i], 6, msg)



    def anglePairTestZ(self, theta_deg, phi_deg):
        ct = np.cos( np.radians(theta_deg))
        st = np.sin( np.radians(theta_deg))

        cp = np.cos( np.radians(phi_deg))
        sp = np.sin( np.radians(phi_deg))


        e1 = sp
        e2 = 0
        e3 = cp

        z = np.array( [0,0,1])
        exp = np.array( [e1, e2, e3] )

        Rz = r.rotateInZMat(theta_deg)
        Ry = r.rotateInYMat(phi_deg)

        rotMat = np.dot(Ry, Rz)
        calc = np.dot(rotMat, z)

        msg = "Angles %.0f %.0f: Expect %s, Calc %s" \
            %(theta_deg, phi_deg, exp, calc)
        for i in range(3):
            self.assertAlmostEqual(exp[i], calc[i], 6, msg)




if __name__ == "__main__":
    unittest.main()




"""Tests K2fov.plot"""

# from .. import plot

"""
def test_basics():
    # Make sure this runs without exception.
    try:
        import matplotlib
        plot.create_context_plot(180, 0)
        plot.create_context_plot_zoomed(180, 0)
    except ImportError:
        pass
"""



#! /usr/bin/env python
# A simple script that will call the K2fov.K2onSilicon() function
import sys

try:
    from K2fov.K2onSilicon import K2onSilicon_main
except ImportError:
    print('You need K2fov installed: pip install K2fov')
    sys.exit(1)

if __name__ == '__main__':
    K2onSilicon_main()



#!/usr/bin/env python
import sys
import os

# Prepare and send a new release to PyPI
if "release" in sys.argv[-1]:
    os.system("python setup.py sdist")
    os.system("twine upload dist/K2fov*")
    os.system("rm -rf dist/K2fov*")
    sys.exit()

try:
    from setuptools import setup

    setup
except ImportError:
    from distutils.core import setup

    setup

# Load the __version__ variable without importing the package
exec(open("K2fov/version.py").read())

# Command-line tools; we're not using "entry_points" for now due to
# a bug in pip which turns all the tools into lowercase
scripts = [
    "scripts/K2onSilicon",
    "scripts/K2findCampaigns",
    "scripts/K2findCampaigns-byname",
    "scripts/K2findCampaigns-csv",
    "scripts/K2inMicrolensRegion",
]

setup(
    name="K2fov",
    version=__version__,
    description="Find which targets are in the field of view of K2",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Fergal Mullally, Tom Barclay, Geert Barentsen",
    author_email="keplergo@mail.arc.nasa.gov",
    url="https://github.com/KeplerGO/K2fov",
    packages=["K2fov"],
    package_data={"K2fov": ["data/*.json"]},
    install_requires=["numpy>=1.8"],
    scripts=scripts,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Astronomy",
    ],
)



"""These tests ensure that the K2 pointing information that was hardcoded
into K2fov in version 1.9.2 is consistent with the info provided by the
'k2-campaigns.json' file that was introduced after.

To run these tests, simply run "py.test" in the K2fov source tree.
"""
from .. import getFieldNumbers, getFieldInfo
from ..K2onSilicon import getRaDecRollFromFieldnum
from ..fields import _getCampaignDict


def old_getRaDecRollFromFieldnum(fieldnum):
    """The code below is copied from K2fov v1.9.2,
    i.e. before the JSON file was added."""
    if fieldnum == 0:
        ra_deg = 98.2964079
        dec_deg = 21.5878901
        scRoll_deg = 177.4810830
    elif fieldnum == 1:
        ra_deg = 173.939610
        dec_deg = 1.4172989
        scRoll_deg = 157.641206
    elif fieldnum == 2:
        ra_deg = 246.1264
        dec_deg = -22.4473
        scRoll_deg = 171.2284
    elif fieldnum == 3:
        ra_deg = 336.66534641438909
        dec_deg = -11.096663792177043
        scRoll_deg = -158.49481806598479
    elif fieldnum == 4:
        ra_deg = 59.0759116
        dec_deg = 18.6605794
        scRoll_deg = -167.6992793
    elif fieldnum == 5:
        ra_deg = 130.1576478
        dec_deg = 16.8296140
        scRoll_deg = 166.0591297
    elif fieldnum == 6:
        ra_deg = 204.8650344
        dec_deg = -11.2953585
        scRoll_deg = 159.6356000
    elif fieldnum == 7:
        ra_deg = 287.82850661398538
        dec_deg = -23.360018153291808
        scRoll_deg = -172.78037532313485
    elif fieldnum == 8:
        ra_deg = 16.3379975
        dec_deg = 5.2623459
        scRoll_deg = -157.3538761
    elif fieldnum == 9:
        ra_deg = 270.3544823
        dec_deg = -21.7798098
        scRoll_deg = 0.4673417
    elif fieldnum == 10:
        ra_deg = 186.7794430
        dec_deg = -4.0271572
        scRoll_deg = 157.6280500
    else:
        raise NotImplementedError

    return (ra_deg, dec_deg, scRoll_deg)


def test_coordinates_file():
    """Are the coordinates in the "k2-campaigns.json" file identical
    to those that were hardcoded in v1.9.2?"""
    campaign_dict = _getCampaignDict()
    campaigns = list(range(0, 11))
    for c in campaigns:
        test_ra, test_dec, test_roll = old_getRaDecRollFromFieldnum(c)
        jsoninfo = campaign_dict["c{0}".format(c)]
        assert(jsoninfo['ra'] == test_ra)
        assert(jsoninfo['dec'] == test_dec)
        assert(jsoninfo['roll'] == test_roll)


def test_coordinates_function():
    """Are the coordinates in the "k2-campaigns.json" file identical
    to those that were hardcoded in v1.9.2?"""
    campaigns = list(range(0, 11))
    for c in campaigns:
        test_ra, test_dec, test_roll = old_getRaDecRollFromFieldnum(c)
        ra, dec, roll = getRaDecRollFromFieldnum(c)
        assert(ra == test_ra)
        assert(dec == test_dec)
        assert(roll == test_roll)


def test_meta_data():
    """Are all the expected meta data fields configured for each campaign?"""
    for fieldnum in getFieldNumbers():
        info = getFieldInfo(fieldnum)
        for key in ["ra", "dec", "roll", "start", "stop"]:
            assert(key in info)
        assert(info["campaign"] == fieldnum)


def test_field_numbers():
    """Is the list of field numbers present and complete?"""
    field_numbers = getFieldNumbers()
    # At the time of writing the test, fields 0 through 16 were defined
    for campaign in range(0, 17):
        assert(campaign in field_numbers)


if __name__ == "__main__":
    test_coordinates()




import numpy as np
import unittest
import os

from .. import fov

#$Id: test_fov.py 49 2014-02-26 17:52:04Z fergalm $
#$URL: svn+ssh://fergalm@svn.code.sf.net/p/keplertwowheel/code/py/test/test_fov.py $




class TestFov(unittest.TestCase):

    def testOrigin(self):
        """Test orientation of FOV in null position"""

        f = fov.KeplerFov(0,0,0)
        #Ch 43 is 13.3, near centre
        a,d = f.getRaDecForChannelColRow(43, 1000, 1000)
        self.assertTrue( np.fabs(a) < .2)
        self.assertTrue( np.fabs(d) < .2)

        #Check that mod 3 is north of mod 8
        a3, d3 = f.getRaDecForChannelColRow(5,0,0)
        a8, d8 = f.getRaDecForChannelColRow(21,0,0)
        self.assertTrue( d3>d8)

        #Check that mod 11's ra is greater than mod 13
        a13, d13 = f.getRaDecForChannelColRow(43,0,0)
        a11, d11 = f.getRaDecForChannelColRow(33,0,0)
        self.assertTrue( a11 > a13)

        #Check that mod 20's ra is less than mod 13
        a13, d13 = f.getRaDecForChannelColRow(43,0,0)
        a20, d20 = f.getRaDecForChannelColRow(69,0,0)
        self.assertTrue(a20 > 350)
        self.assertTrue( a20-360 < a13)


    def testRotateClockwise(self):
        """Test orientation of FOV in null position"""

        f = fov.KeplerFov(0,0,90)
        #Ch 43 is 13.3, near centre
        a,d = f.getRaDecForChannelColRow(43, 1000, 1000)
        self.assertTrue( np.fabs(a-360) < .2)
        self.assertTrue( np.fabs(d) < .2)

        #Check that mod 3's ra less than mod 8
        a3, d3 = f.getRaDecForChannelColRow(5,0,0)
        a8, d8 = f.getRaDecForChannelColRow(21,0,0)
        self.assertTrue(a3 > 350)
        self.assertTrue(a3 < a8)

        #Check that mod 11 is north of mod 13
        a13, d13 = f.getRaDecForChannelColRow(43,0,0)
        a11, d11 = f.getRaDecForChannelColRow(33,0,0)
        self.assertTrue( d11 >  d13)

        #Check that mod 20's dec is less than mod 13
        a13, d13 = f.getRaDecForChannelColRow(43,0,0)
        a20, d20 = f.getRaDecForChannelColRow(69,0,0)
        self.assertTrue( d20 < d13)


    def testQuarter1(self):
        """Test that my code gets the mod-out right for a set of
        stars in Q1.

        This ensures I have the order of the  modouts correct and I didn't
        get anything flipped."""

        #Recreate Kepler pointing in Q1. The source of the 33 degrees
        #for the rotation to get the FOV correct is lost in the mists
        #of time. The 90 just rotates the FOV to the correct angle.
        #For reference, the first light image has the same orientation
        #as Q1.
        a0, d0 = 290.66666667, 44.5
        kf = fov.KeplerFov(a0, d0, 33+90)

        #Find path to the test file
        path = os.path.dirname(os.path.abspath(__file__))
        data = np.loadtxt(os.path.join(path, "data", "Q1_obj_position.txt"))

        for row in data:
            #import pdb; pdb.set_trace()
            ra, dec = row[1:3]
            expectedCh = int(row[5])

            msg = "Uh Oh"
            calcCh = int(kf.pickAChannel(ra, dec))
            self.assertEqual(expectedCh, calcCh, msg)


    def testGitHubBug1(self):
        #Approx coords of field 1
        a0, d0, rho0 = 174., 1.422, 260.6
        kf = fov.KeplerFov(a0, d0, rho0)

        #I'm just checking that the code returns a legal value
        #I don't care about the input.
        ch, col, row = kf.getChannelColRow(+181.946541, 2.740250)

        self.assertRaises(ValueError, kf.getChannelColRow, +1.946541, 2.740250)
        self.assertRaises(ValueError, kf.getChannelColRow, -1.946541, 2.740250)


    def testGitHubBug2(self):
        """Check that -ve ra and dec handled correctly"""
        a0, d0, rho0 = 0., 0, 0
        kf = fov.KeplerFov(a0, d0, rho0)

        #I'm just checking that the code returns a legal value
        #I don't care about the input.
        ch, col, row = kf.getChannelColRow(+1, 0)
        self.assertEqual(ch, 43)

        ch, col, row = kf.getChannelColRow(-1, 0)
        self.assertEqual(ch, 42)

if __name__ == "__main__":
    unittest.main()





import unittest
import numpy as np
from .. import rotate as r

#$Id: test_rotate.py 5 2013-10-29 20:27:09Z fergalm $
#$URL: svn+ssh://fergalm@svn.code.sf.net/p/keplertwowheel/code/py/test/test_rotate.py $#


class TestVecFromRaDec(unittest.TestCase):
    def testRa1(self):
        raList = [0, 90, 180, 270, 360]
        eList = [ [1,0,0],
                  [0,1,0],
                  [-1,0,0],
                  [0,-1,0],
                  [1,0,0]
                ]

        for ra, expect in zip(raList, eList):
            calc = r.vecFromRaDec(ra,0)
            msg = "Expected %s, Calculated %s" %(expect, calc)
            for i in range(3):
                self.assertAlmostEqual(expect[i], calc[i], 10, msg)

    def testRa2(self):

        calc = r.vecFromRaDec(45,0)
        v = np.pi/4.
        expect = [np.cos(v),np.sin(v),0]
        msg = "Expected %s, Calculated %s" %(expect, calc)
        for i in range(3):
            self.assertAlmostEqual(expect[i], calc[i], 10, msg)


    def testDec1(self):
        v = np.sqrt(3)/2.
        decList = [-90, -30, 0, 30, 90]
        eList = [ [0,0,-1],
                  [v, 0, -.5] ,
                  [1,0,0],
                  [v, 0, +.5],
                  [0,0,1],
                ]

        for dec, expect in zip(decList, eList):
            calc = r.vecFromRaDec(0, dec)
            msg = "Expected %s, Calculated %s" %(expect, calc)
            for i in range(3):
                self.assertAlmostEqual(expect[i], calc[i], 10, msg)

    def testRaDec1(self):
        calc = r.vecFromRaDec(45, 60)
        a = 1/(2*np.sqrt(2))
        b = np.sqrt(3)/2.
        expect = [a, a, b]

        msg = "Expected %s, Calculated %s" %(expect, calc)
        for i in range(3):
            self.assertAlmostEqual(expect[i], calc[i], 10, msg)

    def testRaDec2(self):
        calc = r.vecFromRaDec(135, 60)
        a = 1/(2*np.sqrt(2))
        b = np.sqrt(3)/2.
        expect = [-a, a, b]

        msg = "Expected %s, Calculated %s" %(expect, calc)
        for i in range(3):
            self.assertAlmostEqual(expect[i], calc[i], 10, msg)


    def testKeplerBoresight(self):
        calc = r.vecFromRaDec(290.666667, 44.5)
        expect = [.2517, -.6674, .7009]

        msg = "Expected %s, Calculated %s" %(expect, calc)
        for i in range(3):
            self.assertAlmostEqual(expect[i], calc[i], 4, msg)



class TestRaDecFromVec(unittest.TestCase):
    def testRa1(self):
        eList = [0, 90, 180, 270]
        vecList = [ [1,0,0],
                    [0,1,0],
                    [-1,0,0],
                    [0,-1,0],
                ]

        for vec, expect in zip(vecList, eList):
            calc = r.raDecFromVec(vec)
            msg = "Expected %s, Calculated %s" %(expect, calc[0])
            self.assertAlmostEqual(expect, calc[0], 10, msg)

    def testRa2(self):
        a = .5
        b = np.sqrt(3)/2.
        eList = [30, 60, 120, 150, 210, 240, 300, 330]
        vecList = [ [b, a, 0],
                    [a, b, 0],
                    [-a, b, 0],
                    [-b, a, 0],
                    [-b, -a, 0],
                    [-a, -b, 0],
                    [a, -b, 0],
                    [b, -a, 0],
                ]

        for vec, expect in zip(vecList, eList):
            calc = r.raDecFromVec(vec)
            msg = "Expected %s, Calculated %s" %(expect, calc[0])
            self.assertAlmostEqual(expect, calc[0], 10, msg)



class TestRotationAboutAxis(unittest.TestCase):
    """Extensive testing of rotateInRa and rotateInDec
    (and by extension rotateAroundVector)
    """

    def testRotateInRa1(self):
        a = np.array([1,0,0])
        b = r.rotateInRa(a, 90)

        expect = [0,1,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)

    def testRotateInRa2(self):
        a = np.array([0,1,0])
        b = r.rotateInRa(a, 90)

        expect = [-1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)

    def testRotateInRa3(self):
        a = np.array([0,0,1])
        b = r.rotateInRa(a, 90)

        expect = [0,0,1]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)

    def testRotateInRa4(self):
        alpha = 285 * np.pi/180.
        a = np.array([np.cos(alpha), np.sin(alpha),0])
        b = r.rotateInRa(a, -285)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)

    def testRotateInRa4b(self):
        a = r.vecFromRaDec(285, 0)
        b = r.rotateInRa(a, -285)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)


    def testRotateInRa5(self):
        a = r.vecFromRaDec(0, 44)
        b = r.rotateInRa(a, 10)

        new = r.raDecFromVec(b)
        expect = [10.,44.]
        msg = "Expected %s, Calculated %s" %(expect, new)
        for i in range(2):
            self.assertAlmostEqual(expect[i], new[i], 10, msg)



    def testRotateInDec1(self):
        a = np.array([1,0,0])
        b = r.rotateInDeclination(a, 90)

        expect = [0,0,+1]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)


    def testRotateInDec2(self):
        a = np.array([0,1,0])
        b = r.rotateInDeclination(a, 90)

        expect = [0,1,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)


    def testRotateInDec3(self):
        a = np.array([0,0,1])
        b = r.rotateInDeclination(a, -90)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)


    def testRotateInDec4(self):
        a = r.vecFromRaDec(0, 44)
        b = r.rotateInDeclination(a, -44)
        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10, msg)

    def testRotateInRaDec(self):
        a = r.vecFromRaDec(285, 44)
        b = r.rotateInRa(a, -285)
        c = r.rotateInDeclination(b, -44)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, c)
        for i in range(3):
            self.assertAlmostEqual(expect[i], c[i], 10, msg)



class TestRotationMatrix(unittest.TestCase):
    def testRa1(self):
        a = np.array([1,0,0])
        R = r.rightAscensionRotationMatrix(90.)
        b = np.dot(R, a)

        expect = [0,1,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)

    def testRa2(self):
        a = r.vecFromRaDec(285, 0)
        R = r.rightAscensionRotationMatrix(-285)
        b = np.dot(R, a)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)

    def testRa2(self):
        a = r.vecFromRaDec(0, 45)
        R = r.rightAscensionRotationMatrix(30)
        b = np.dot(R, a)
        new = r.raDecFromVec(b)

        expect = [30, 45]
        msg = "Expected %s, Calculated %s" %(expect, new)
        for i in range(2):
            self.assertAlmostEqual(expect[i], new[i], 10)


    def testDec1(self):
        a = np.array([1,0,0])
        R = r.declinationRotationMatrix(90.)
        b = np.dot(R, a)

        expect = [0,0,1]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)


    def testDec2(self):
        a = r.vecFromRaDec(0, 45)
        R = r.declinationRotationMatrix(-45)
        b = np.dot(R, a)

        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)

    def testRaDec(self):
        a = r.vecFromRaDec(285, 45)
        Ra = r.rightAscensionRotationMatrix(-285)
        Rd = r.declinationRotationMatrix(-45)

        b = np.dot(Rd, np.dot(Ra, a))
        expect = [1,0,0]
        msg = "Expected %s, Calculated %s" %(expect, b)
        for i in range(3):
            self.assertAlmostEqual(expect[i], b[i], 10)




if __name__ == "__main__":
    unittest.main()






import unittest
from .. import projection as proj

#$Id: test_projection.py 40 2014-02-18 20:59:31Z fergalm $
#$URL: svn+ssh://fergalm@svn.code.sf.net/p/keplertwowheel/code/py/test/test_projection.py $#


class TestGnomic(unittest.TestCase):
    def testSkyToPixAtOrigin(self):

        """Test that increasing ra causes a decreasing pixel position,
        but that an increasing dec causes an increasing pixel position"""

        arcsec = 1/3600.
        p = proj.Gnomic(0,0)

        x, y = p.skyToPix(0,0)
        msg = "Expected 0,0, got %.3f %.3f" %(x, y)
        self.assertAlmostEqual(0, x[0], 10, msg)
        self.assertAlmostEqual(0, y[0], 10, msg)

        x, y = p.skyToPix(arcsec,0)
        msg = "Expected -,0, got %.6f %.6f" %(x, y)
        self.assertTrue(x[0]<0, msg)
        self.assertAlmostEqual(0, y[0], 10, msg)

        x, y = p.skyToPix(-arcsec,0)
        msg = "Expected +,0, got %.6f %.6f" %(x, y)
        self.assertTrue(x[0]>0, msg)
        self.assertAlmostEqual(0, y[0], 10, msg)

        x, y = p.skyToPix(0, arcsec)
        msg = "Expected 0,+, got %.6f %.6f" %(x, y)
        self.assertAlmostEqual(0, x[0], 10, msg)
        self.assertTrue(y[0]>0, msg)

        x, y = p.skyToPix(0, -arcsec)
        msg = "Expected 0,-, got %.6f %.6f" %(x, y)
        self.assertAlmostEqual(0, x[0], 10, msg)
        self.assertTrue(y[0]<0, msg)

        x, y = p.skyToPix(arcsec, arcsec)
        msg = "Expected -,+, got %.6f %.6f" %(x, y)
        self.assertTrue(x[0]<0, msg)
        self.assertTrue(y[0]>0, msg)

        x, y = p.skyToPix(arcsec, -arcsec)
        msg = "Expected -,-, got %.3e %.3e" %(x, y)
        self.assertTrue(x[0]<0, msg)
        self.assertTrue(y[0]<0, msg)


    def testSkyToPixAtPosition(self):
        arcsec = 1/3600.
        a = 36.
        d = 14.
        p = proj.Gnomic(a, d)

        x, y = p.skyToPix(a,d)
        msg = "Expected 0,0, got %.3f %.3f" %(x, y)
        self.assertAlmostEqual(0, x[0], 10, msg)
        self.assertAlmostEqual(0, y[0], 10, msg)

        x, y = p.skyToPix(a+arcsec,d)
        msg = "Expected +,0, got %.3f %.3f" %(x, y)
        self.assertTrue(x[0]<0, msg)
        self.assertAlmostEqual(0, y[0], 10, msg)

        x, y = p.skyToPix(a+arcsec,d+arcsec)
        msg = "Expected +,0, got %.3f %.3f" %(x, y)
        self.assertTrue(x[0]<0, msg)
        self.assertTrue(y[0]>0, msg)


    def testPixToSkyAtOrigin(self):
        arcsec = 1/3600.
        p = proj.Gnomic(0,0)

        a, d = p.pixToSky(0,0)
        msg = "Expected 0,0, got %.3f %.3f" %(a, d)
        self.assertAlmostEqual(0, a[0], 10, msg)
        self.assertAlmostEqual(0, d[0], 10, msg)

        a, d = p.pixToSky(arcsec,0)
        msg = "Expected +,0, got %.6f %.6f" %(a, d)
        self.assertTrue(a[0]>359, msg)
        self.assertAlmostEqual(0, d[0], 10, msg)

        a, d = p.pixToSky(-arcsec,0)
        msg = "Expected -,0, got %.6f %.6f" %(a, d)
        self.assertTrue(a[0]<1, msg)
        self.assertAlmostEqual(0, d[0], 10, msg)

        a, d = p.pixToSky(0, arcsec)
        msg = "Expected 0,+, got %.6f %.6f" %(a, d)
        self.assertAlmostEqual(0, a[0], 10, msg)
        self.assertTrue(d[0]>0, msg)

        a, d = p.pixToSky(0, -arcsec)
        msg = "Expected 0,-, got %.6f %.6f" %(a, d)
        self.assertAlmostEqual(0, a[0], 10, msg)
        self.assertTrue(d[0]<360, msg)



    def testPixToSkyAtPosition(self):
        a0 = 36
        d0 = 14
        tol = 6
        arcsec = 1/3600.
        p = proj.Gnomic(a0, d0)

        a, d = p.pixToSky(0,0)
        msg = "Expected 0,0, got %.3f %.3f" %(a, d)
        self.assertAlmostEqual(a[0], a0, 10, msg)
        self.assertAlmostEqual(d[0], d0, 10, msg)

        a, d = p.pixToSky(arcsec,0)
        msg = "Expected -,0, got %.9f %.9f" %(a, d)
        self.assertTrue(a[0]<a0, msg)

        a, d = p.pixToSky(-arcsec,0)
        msg = "Expected +,0, got %.9f %.9f" %(a, d)
        self.assertTrue(a[0]>a0, msg)

        a, d = p.pixToSky(0, arcsec)
        msg = "Expected 0,+, got %.9f %.9f" %(a, d)
        self.assertAlmostEqual(a[0], a0, tol, msg)
        self.assertTrue(d[0]>d0, msg)

        a, d = p.pixToSky(0, -arcsec)
        msg = "Expected 0,-, got %.9f %.9f" %(a, d)
        self.assertAlmostEqual(a[0], a0, tol, msg)
        self.assertTrue(d[0]<d0, msg)


    def testUnprojectable(self):
        """Test that unprojectable points are caught"""
        pg = proj.Gnomic(0,0)

        #Not interested in the  value, only that the answer is returned
        for i in range(-89, 90):
            pg.skyToPix(i,0)

        #I couldn't get self.assertRaises to work for me.
        for i in range(-180, -90):
            try:
                pg.skyToPix(i,0)
            except ValueError:
                pass
            else:
                self.assertTrue(False, "skyToPix didn't throw an exception when it should")

        for i in range(91, 180):
            try:
                pg.skyToPix(i,0)
            except ValueError:
                pass
            else:
                self.assertTrue(False, "skyToPix didn't throw an exception when it should")



if __name__ == "__main__":
    unittest.main()


